%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%          Active Magentic Regenerative Refrigerator Model           %%
%%          developed at the University of Wisconsin - Madison       %%
%%              calls regenerator model version 3_6                          %%
%%                                                                                           %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% This input module takes modeling inputs including fluid flow and magnetic
% field profiles and passes them to the regenerator model.  Fluid flow rate
% and magnetic field are determined using functions specified in this
% script.  The outputs of the regenerator model include the temperature
% profile of the fluid and regnerator once cyclical steady state has been
% achieved as well as pumping power and heating and cooling loads.

clear %clears the existing variable space

%Volume=.033; %L, total volume of the regenerator material
nbed=6; %number of beds in the regenerator
h=.02; %m, height of flow channel in regenerator
w=.02; %m, width of flow channel in regenerator
Ac=h*w; %m2, cross sectional area of the regenerator
L=.06; %m, length of each regenerator bed
bed_vol=Ac*L; %volume per bed
dr=1/3; %mass flow rate dwell ratio
unbal=0.0; %flow unbalance in regenerator
flow_ramp=0.1; %ramp period for fluid flow
muoH_max=1.5; %Tesla, maximum magnetic field generated by the magnetic source

%inputs associated with the regenerator packing
pacRAW=[0.36, 0, .0005, 0]; %[eps, dummy, dp, dummy] for packed sphere regenerator

%inputs associated with the magnetic field profile
maginp=[120,15,360/nbed,2,L];%[arc of magnet, arc of ramp, arc of each bed, bed type, regenerator length]


%% Model parameters

Nx=60; %number of spatial steps
Nt=4000; %number of time steps
tol=.0002; %convergence tolerance

%% Define the user-defined functions called to determine properties

magpropfun=@Gdcom; %function called to determine magnetocaloric materials
fluidfun=@props_water; %the fluid is a 10% ethylene glycol and water mixture
bedcorrfun=@SphPart2; %regenerator correlation function
muoHfun=@muoH_ramp2; %ramped field function
mcmpropfun=@Gdcom; %magnetocaloric material property function
massflowfun=@mdotsmoothunbal_delay; %fluid flow function

%%

modelcheck=1; %1=plot model results, 0=do not plot results


TC=285; %K
TH=300; %K
tau=0.5; %sec, cycle duration
delay=15/360*tau; %sec, delay time in seconds from the beginning of the cycle before the blow period begins
mdot_amp=.03; %kg/s,   mass flow rate amplitude
massflow_inp=[dr,delay,flow_ramp,unbal]; %inputs associated with the fluid mass flow function
    
%% determine mass flow and magnetic field profiles

ii=1:Nx;
jj=0:Nt;
t_node=(jj(1:Nt)'+0.5)/Nt*tau; %time grid at center of time step
xr=L*(ii'-1/2)/Nx; %regenerator spatial grid
mdot=massflowfun(t_node,tau,mdot_amp,massflow_inp); %determines the mass flow of fluid vs time
muoH=muoHfun(xr,t_node,tau,muoH_max,maginp); %determines magnetic field vs time
muoH(:,Nt+2)=muoH(:,1); %fills beginning of next cycle muoH at end of muoH array


%% Call the AMR bed model

[Qpump,ref_cap,heat_rej,Tr,Tf,mdot,muoH]=AMRregen_3_6(L,Ac,tau,TC,TH,Nx,Nt,nbed,tol,mdot_amp,dr,pacRAW,mcmpropfun,fluidfun,bedcorrfun,modelcheck,mdot,muoH);

fprintf('Finsihed\n\n')


