%% This is the main regenerator model developed at UW.  This function takes
%% inputs that define the system and forcing functions, such as mass flow
%% profile, and calculates the regenerator performance.  This model does
%% not account for heat exchangers.  The main outputs of the model are the
%% cyclical steady state temperatures of the fluid and solid over an entire
%% cycle and the heat rejection, cooling power, and pumping power of the
%% system.  This model is described in Progress Report #1.
%%
%%This model is based on version 1_2_4, which was developed at Astronautics
%%over the summer of 2006.  It was brought back to UW on August 28, 2006.
%% 3_2 changes the indexing slighttly so that the number of rows in the 
%% temperature matrices as the time array.
%% 3_3 uses the heat transfer degradation factor from the Journal of Heat
%% Transfer paper and fixed bug in calculation of t_node
%% 3_4 the inputs to this function are modified to allow more flexibility
%% in the text input file.  The call to the magnetic field function has
%% been modified to be more flexible.
%% 3_5 accepts the mass flow rate and magnetic field as inputs to the
%% function.  These values are no longer calculated inside the function as
%% previous functions did.
%% 3_6 added additional outputs from the function, removed some unused
%% lines of code and other housekeeping.

function[Qpump,ref_cap,heat_rej,Tr,Tf,mdot,muoH]=AMRregen_3_6(L,Ac,tau,TC,TH,Nx,Nt,n_beds,tol,mdot_amp,dwell,pacRAW,magpropfun,fluidfun,bedcorrfun,modelcheck,mdot,muoH);
                                                                        
%%Bed paramters

tic %start timer to determine clock time needed to achieve convergence
dby=0; %m, bypass flow pipe diameter

[dh,eps,epsX,epsY,as,FAR,PAR]=pack_geom(pacRAW,bedcorrfun);
n_motor=.9; %electric motor efficiency
n_pump=.7; %pump efficiency
imp=1; %1=run implicit to generate guess values, 0=don't run implicit

%%Conditions
cnk=1; %axial conduction scaling factor
cff=1; %friction factor scaling factor
cNu=1; %Nusselt number scaling factor
layer=0; %0 not layered 1=layered

extrapnum=3; %number of cycles between extrapolations
extlin=6; %linear number of cycles to extrapolate
order=1; %polynomial fit order
wt=1;

dt=tau/Nt; %time step in s

%% Allocate the sparse matrices A and B.
N=2*Nx;
A=spalloc(N,N,8*N);
B=spalloc(N,1,N);
%%

i=1:Nx;
xf=L*(i'-1/2)/Nx; %fluid spatial grid
xr=xf; %regenerator spatial grid
j=0:Nt;
t=j'*tau/(Nt);
t_node=(j(1:Nt)'+0.5)/Nt*tau; %time grid at center of time step
dt=t(2)-t(1);
TCurie=292*ones(Nx,1);

if imp==0
    
    %% start model with a ramp temperature profile
    
    Tr(:,1)=TH-xr*(TH-TC)/L; %initial regenerator temp
    Tf(:,1)=TH-xf*(TH-TC)/L;  %initial fluid temp
else

    %% call implicit UW model to generate initial guess values and speed
    %% model convergence
    
    pacCOOKED=[pacRAW,epsX,epsY,as,eps,dh,FAR,PAR,cNu,cff,cnk];
    [Trg,Tfg,tf,tr]=AMR_impguessT_5(magpropfun,fluidfun,bedcorrfun,pacCOOKED,L,tau,Ac,Nx,mdot_amp,t_node,TH,TC,TCurie,n_beds,mdot,muoH);
    Tr(:,1)=Trg(:,1);
    Tf(:,1)=Tfg(:,1);
end

%% inputs for new mcmprops

magpropfun=@mcmprops;
numlayer=1; %number of layers in the regenerator
mcm_inp=[294, 1]; 
mcmfun={@Gdcom}; %property function used for each layer.  This array must use the curly brackets {} to stay compatible with other matlab versions

%calculate energy difference between fluid at TH and TC.  These properties
%are also used to calculated heat flux at the hot and cold reservoirs.
[mufC,kfC,cfC,rhofC,hfC,sfC]=fluidfun(TC);
[mufH,kfH,cfH,rhofH,hfH,sfH]=fluidfun(TH);
deltaUf=hfH-hfC;

% initialize variables before beginning the main loop.  These variables are
% used to check for co
done=0;
j=0;
ref_load=[-500 -500];
Er=[0 0]; %this matrix stores the total energy in the fluid for this past two cycles
Ef=[0 0];%this matrix stores the total energy in the solid material for this past two cycles
Etot=[0 0];%this matrix stores the total energy in the fluid and solid for this past two cycles
count=0;
count2=0;
iteration(1)=1;
deltaE=zeros(extrapnum,1);

while(done==0)
    j=j+1;
    time=t(j);
    [muf(:,j),kf(:,j),cf(:,j),rhof,hf(:,j),sf(:,j)]=fluidfun(Tf(:,j));
    v=-mdot/(Ac*rhof);
    Prf(:,j)=cf(:,j).*muf(:,j)./(kf(:,j)); %Prandtl number
    Ref(:,j)=4*abs(v(j))*rhof./(as*muf(:,j)); %Reynolds number based on hydraulic diameter
    [cmuoH(:,j),dsdmuoH(:,j),rhor,kr(:,j),sr(:,j),hr(:,j)]=magpropfun(xr,L,Tr(:,j),muoH(:,j),numlayer,mcm_inp,mcmfun);
    cf_av=mean(mean(cf)); %average fluid specific heat over the entire cycle
    muf_av=mean(mean(muf)); %average fluid viscosity over the entire cycle
    cmuoH_av=mean(mean(cmuoH)); %average solid specific heat over the entire cycle

    pacCOOKED=[pacRAW,epsX,epsY,as,eps,dh,FAR,PAR,cNu,cff,cnk]; %regenerator parameters that are generalized in terms of hydraulic diameter, etc.
    [Nuf(:,j),ff(:,j),kstat(:,j),kdisp(:,j)]=bedcorrfun(pacCOOKED,Ref(:,j),Prf(:,j),kf(:,j), kr(:,j),v,muf(:,j),rhof,cf(:,j),tau);
    dP(:,j)=ff(:,j).*(Ref(:,j)).^2*2/dh^3.*(muf(:,j)/rhof).^2*rhof*L/Nx; %Pressure drop in Pa, based on hydraulic diameter

    %% Fill the A and B matrices according to the system governing
    %% equations.  A and B are used to solve for the temperature of every
    %% node in the regenerator at each time step in the cycle.
    
    if(mdot(j)>=0)
        %hot-to-cold flow
        % fluid energy equations
        for i=2:Nx-1;
            h(i,j)=Nuf(i,j).*kf(i,j)/dh; %convection coefficient in node i,j
            A(i,i)=eps*rhof*Ac*L/Nx*cf(i,j)/dt+h(i,j)*as*Ac*L/Nx+2*kdisp(i,j)*Ac*Nx/L; %energy balance for center node
            A(i,i-1)=-kdisp(i,j)*Ac*Nx/L-mdot(j)*cf(i,j)/2; %energy balance for left node
            A(i,i+1)=-kdisp(i,j)*Ac*Nx/L+mdot(j)*cf(i,j)/2; %energy balance for right node
            A(i,Nx+i)=-h(i,j)*as*Ac*L/Nx; %energy balance for regenerator
            B(i)=Tf(i,j)*eps*rhof*Ac*L/Nx*cf(i,j)/dt+dP(i,j)*abs(mdot(j))/rhof; %right hand side of energy balance
        end
        i=1; %hot end
        h(i,j)=Nuf(i,j).*kf(i,j)/dh;
        A(i,i)=eps*rhof*Ac*L/Nx*cf(i,j)/dt+cf(i,j)*mdot(j)/2+h(i,j)*as*Ac*L/Nx+1*kdisp(i,j)*Ac*Nx/L;
        A(i,i+1)=-kdisp(i,j)*Ac*Nx/L+mdot(j)*cf(i,j)/2;
        A(i,Nx+i)=-h(i,j)*as*Ac*L/Nx;
        B(i)=Tf(i,j)*eps*rhof*Ac*L/Nx*cf(i,j)/dt+dP(i,j)*abs(mdot(j))/rhof+TH*cf(i,j)*mdot(j);
        i=Nx; %cold end
        h(i,j)=Nuf(i,j).*kf(i,j)/dh;
        A(i,i)=eps*rhof*Ac*L/Nx*cf(i,j)/dt+cf(i,j)*mdot(j)+h(i,j)*as*Ac*L/Nx+1*kdisp(i,j)*Ac*Nx/L;
        A(i,i-1)=-kdisp(i,j)*Ac*Nx/L-mdot(j)*cf(i,j);
        A(i,Nx+i)=-h(i,j)*as*Ac*L/Nx;
        B(i)=Tf(i,j)*eps*rhof*Ac*L/Nx*cf(i,j)/dt+dP(i,j)*abs(mdot(j))/rhof;

    else
        if(mdot(j)<0)
            %cold-to-hot flow
            % fluid energy equations
            for i=2:Nx-1;
                h(i,j)=Nuf(i,j).*kf(i,j)/dh; %convection coefficient in node i,j
                A(i,i)=eps*rhof*Ac*L/Nx*cf(i,j)/dt+h(i,j)*as*Ac*L/Nx+2*kdisp(i,j)*Ac*Nx/L; %energy balance for center node
                A(i,i-1)=-kdisp(i,j)*Ac*Nx/L-mdot(j)*cf(i,j)/2; %energy balance for right node
                A(i,i+1)=-kdisp(i,j)*Ac*Nx/L+mdot(j)*cf(i,j)/2; %energy balance for right node
                A(i,Nx+i)=-h(i,j)*as*Ac*L/Nx; %energy balance for regenerator
                B(i)=Tf(i,j)*eps*rhof*Ac*L/Nx*cf(i,j)/dt+dP(i,j)*abs(mdot(j))/rhof; %right hand side of energy balance
            end
            i=1; %hot end
            h(i,j)=Nuf(i,j).*kf(i,j)/dh;
            A(i,i)=eps*rhof*Ac*L/Nx*cf(i,j)/dt-cf(i,j)*mdot(j)+h(i,j)*as*Ac*L/Nx+1*kdisp(i,j)*Ac*Nx/L;
            A(i,i+1)=-kdisp(i,j)*Ac*Nx/L+mdot(j)*cf(i,j);
            A(i,Nx+i)=-h(i,j)*as*Ac*L/Nx;
            B(i)=Tf(i,j)*eps*rhof*Ac*L/Nx*cf(i,j)/dt+dP(i,j)*abs(mdot(j))/rhof;
            i=Nx; %cold end
            h(i,j)=Nuf(i,j).*kf(i,j)/dh;
            A(i,i)=eps*rhof*Ac*L/Nx*cf(i,j)/dt-cf(i,j)*mdot(j)/2+h(i,j)*as*Ac*L/Nx+1*kdisp(i,j)*Ac*Nx/L;
            A(i,i-1)=-kdisp(i,j)*Ac*Nx/L-mdot(j)*cf(i,j)/2;
            A(i,Nx+i)=-h(i,j)*as*Ac*L/Nx;
            B(i)=Tf(i,j)*eps*rhof*Ac*L/Nx*cf(i,j)/dt+dP(i,j)*abs(mdot(j))/rhof-TC*mdot(j)*cf(i,j);

        else
            %no flow

        end
    end
    for i=2:Nx-1
        % regenerator energy equations
        A(Nx+i,Nx+i)=Ac*L/Nx*(1-eps)*rhor(i)*cmuoH(i,j)/dt+h(i,j)*as*Ac*L/Nx+2*kstat(i,j)*Ac*Nx/L; %energy equation for center node
        A(Nx+i,Nx+i-1)=-kstat(i,j)*Ac*Nx/L; %energy equation for left node
        A(Nx+i,Nx+i+1)=-kstat(i,j)*Ac*Nx/L; %energy equation for right node
        A(Nx+i,i)=-h(i,j)*as*Ac*L/Nx; %energy equation for fluid 
        B(Nx+i)=Ac*L/Nx*(1-eps)*rhor(i)*cmuoH(i,j)*Tr(i,j)/dt-Ac*L/Nx*(1-eps)*rhor(i)*Tr(i,j)*dsdmuoH(i,j)*(muoH(i,j+1)-muoH(i,j))/dt; %right hand side of energy equation
    end
    i=1; %hot end
    A(Nx+i,Nx+i)=Ac*L/Nx*(1-eps)*rhor(i)*cmuoH(i,j)/dt+h(i,j)*as*Ac*L/Nx+kstat(i,j)*Ac*Nx/L;
    A(Nx+i,Nx+i+1)=-kstat(i,j)*Ac*Nx/L;
    A(Nx+i,i)=-h(i,j)*as*Ac*L/Nx;
    B(Nx+i)=Ac*L/Nx*(1-eps)*rhor(i)*cmuoH(i,j)*Tr(i,j)/dt-Ac*L/Nx*(1-eps)*rhor(i)*Tr(i,j)*dsdmuoH(i,j)*(muoH(i,j+1)-muoH(i,j))/dt;
    i=Nx;
    A(Nx+i,Nx+i)=Ac*L/Nx*(1-eps)*rhor(i)*cmuoH(i,j)/dt+h(i,j)*as*Ac*L/Nx+kstat(i,j)*Ac*Nx/L;
    A(Nx+i,Nx+i-1)=-kstat(i,j)*Ac*Nx/L;
    A(Nx+i,i)=-h(i,j)*as*Ac*L/Nx;
    B(Nx+i)=Ac*L/Nx*(1-eps)*rhor(i)*cmuoH(i,j)*Tr(i,j)/dt-Ac*L/Nx*(1-eps)*rhor(i)*Tr(i,j)*dsdmuoH(i,j)*(muoH(i,j+1)-muoH(i,j))/dt;

    %%invert matrix A to calculate the temperature of the fluid and solid
    %%for the current time step then pull fluid and regenerator
    %%temperatures from the resulting X matrix.
    
    X=A\B; %the matrix X contains the fluid temp in rows 1 to Nx, and regenerator temp in rows Nx + 1 to 2Nx

    %% extract the fluid and solid temps from the sparse matrix X using the
    %% matlab command 'full'

    i=1:Nx; 
    Tf(:,j+1)=full(X(i));
    i=1:Nx;
    Tr(:,j+1)=full(X(Nx+i));
    
    if j==Nt %end of cycle, check for convergence
        %% calculate properties for Nt+1 time step
        j=Nt+1;
        [muf(:,j),kf(:,j),cf(:,j),rhof,hf(:,j),sf(:,j)]=fluidfun(Tf(:,j));
        Prf(:,j)=cf(:,j).*muf(:,j)./(kf(:,j)); %Prandtl number
        [cmuoH(:,j),dsdmuoH(:,j),rhor,kr(:,j),sr(:,j),hr(:,j)]=magpropfun(xr,L,Tr(:,j),muoH(:,j),numlayer,mcm_inp,mcmfun);

        %calculate system performance for previous cycle
        TCedg=0.5*(1.5*Tf(Nx,1:Nt)-0.5*Tf(Nx-1,1:Nt))+0.5*(1.5*Tf(Nx,2:Nt+1)-0.5*Tf(Nx-1,2:Nt+1)); %Fluid temperature at the hot edge of the bed.  This temp is extrapolated from the last two nodes in the bed
        [mufCedg,kfCedg,cfCedg,rhof,hfCedg,sfCedg]=fluidfun(TCedg); %Use the fluid temp at the cold end of the bed to determine fluid properties at the cold end of the bed.
        hflux(Nx+1,1)=sum(mdot.*hfCedg')*tau/Nt; %net enthalpy flux at the cold end of the bed
        ref_load=-hflux(Nx+1); %J
        THedg=0.5*(1.5*Tf(1,1:Nt)-0.5*Tf(2,1:Nt))+0.5*(1.5*Tf(1,2:Nt+1)-0.5*Tf(2,2:Nt+1)); %Fluid temperature at the hot edge of the bed.  This temp is extrapolated from the last two nodes in the bed
        [mufHedg,kfHedg,cfHedg,rhof,hfHedg,sfHedg]=fluidfun(THedg); %Use the fluid temp at the cold end of the bed to determine fluid properties at the hot end of the bed.
        heat_load=-sum(mdot.*hfHedg')*tau/Nt; %J
        %calculate pressure drop in the regenerator and pump work
        for j=1:Nt
            dWpump(:,j)=abs(mdot(j)/rhof*dP(:,j)); %W
            deltaP(j)=sum(dP(:,j));
        end
        Wpump=sum(sum(dWpump,1),2)*dt %pump work in J

        %calculate bypass flow from the regenerator and associated energy
        %losses
        dQby=pi*deltaP*dby^4/(128*muf_av*L);
        Qby=trapz(t(1:(Nt/2+1)),dQby(1:(Nt/2+1)));
        Vby=L*pi/4*dby^2; %bypass fluid volume in the regenerator
        qby=(Qby-Vby)*rhof*deltaUf; %heat loss due to bypass flow
        
        Ef=eps*rhof*hf*Ac*L/Nx; %fluid energy entrained in one bed of the regenerator
        Er=(1-eps)*rhor(1)*hr*Ac*L/Nx; %regenerator energy for one bed
        dE_cycle=sum(abs(Er(:,Nt+1)-Er(:,1)))+sum(abs(Ef(:,Nt+1)-Ef(:,1))) %absolute value of the change in energy over one cycle
        Wmag=dE_cycle-Wpump+heat_load-ref_load; %magnetic work during the previous cycle
        Emax=max(sum(Ef,1)+sum(Er,1)); %maximum energy in the solid and fluid during previous cycle
        Emin=min(sum(Ef,1)+sum(Er,1)); %minimum energy in the solid and fluid during previous cycle
        err=abs(dE_cycle/(Emax-Emin)) %figure of merit for cyclical steady state convergence
        count=count+1
        count2=count2+1;
        if count==1
            E0=Ef(Nt+1)+Er(Nt+1);
        end
        deltaE(count2)=dE_cycle/E0;
        if err<tol %system has reached steady state, exit while loop.
            %         if deltaU<tol
            done=1;
        else %steady state has not been achieved, continue to next cycle.
            j=0; A=0*A; B=0*B; %reset matrices A and B
            cycle=(1:1:extrapnum)';
            dE=mean(deltaE);
            Tr(:,1)=Tr(:,Nt+1); %initial regenerator temp for next cycle
            Tf(:,1)=Tf(:,Nt+1);  %initial fluid temp for next cycle
            Tr_log(:,count)=Tr(:,Nt+1);
            Tf_log(:,count)=Tf(:,Nt+1);
            Tr_log2(:,count2)=Tr(:,Nt+1);
            Tf_log2(:,count2)=Tf(:,Nt+1);
            if count2==extrapnum
              for i=1:Nx
                    P=polyfit(cycle,Tr_log2(i,:)',order);
                    Tlin=polyval(P,extlin);
                    Tr(i,1)=Tr_log2(i,extrapnum)+wt*(Tlin-Tr_log2(i,extrapnum));
                    P=polyfit(cycle,Tf_log2(i,:)',order);
                    Tlin=polyval(P,extlin);
                    Tf(i,1)=Tf_log2(i,extrapnum)+wt*(Tlin-Tf_log2(i,extrapnum));
                end
                Tr_log2=zeros(size(Tr_log2));
                Tf_log2=zeros(size(Tf_log2));
                count2=0;
                deltaE=zeros(extrapnum,1);
            end

        end
        %done=1
    end
    %     done=1
end

TCedg=0.5*(1.5*Tf(Nx,1:Nt)-0.5*Tf(Nx-1,1:Nt))+0.5*(1.5*Tf(Nx,2:Nt+1)-0.5*Tf(Nx-1,2:Nt+1)); %Fluid temperature at the hot edge of the bed.  This temp is extrapolated from the last two nodes in the bed
[mufCedg,kfCedg,cfCedg,rhof,hfCedg,sfCedg]=fluidfun(TCedg);
Tout_avg=trapz(t(1:Nt/2),TCedg(1:(Nt)/2)')/(t((Nt)/2)-t(1))%*pi/(2*mdot_amp)
eff=(TH-Tout_avg)/(TH-TC)
NTU=Nuf(1,1)*kf(1,1)*Ac*L*as/(dh*mdot_amp*cf(1,1));

% dwell=1/3;
% n_beds=6

U=mdot_amp*dwell*cf(1,1)*tau/(2*Ac*L*(1-eps)*rhor(1)*cmuoH(1,1));
Crat=Ac*L*(1-eps)*rhor(1)*cmuoH(1,1)/(mdot_amp*cf(1,1)*tau/2);
count=count
t_conv=toc %this determines how long the model took to converge
 
%%%Calculate dimensionless parameters
V=Ac*L;
cr_avg=mean(mean(cmuoH));
cf_avg=mean(mean(cf));
Cr=mean(rhor)*V*(1-eps)*cr_avg;
Cf=rhof*V*(eps)*cf_avg;
R=Cf/Cr;
lambda=tau/2*(1-dwell);
U=mdot_amp*cf_avg*lambda/(Cr+Cf);
mdot_max=max(mdot);
z=0;
for i=1:size(mdot)
    if mdot(i)==mdot_max
        z=z+1;
        Nu_flow(z)=Nuf(1,i);
    end
end
Nu_avg=mean(Nu_flow);
NTU=Nu_avg*kf(1,19)*Ac*L*as/(dh*mdot_amp*cf_avg);

%% Calculate net enthalpy and entropy fluxes

for i=1:Nx
   hflux(i,1)=sum(mdot.*hf(i,1:Nt)')*tau/Nx;
   sflux(i,1)=sum(mdot.*sf(i,1:Nt)')*tau/Nx;
end
THedg=0.5*(1.5*Tf(1,1:Nt)-0.5*Tf(2,1:Nt))+0.5*(1.5*Tf(1,2:Nt+1)-0.5*Tf(2,2:Nt+1)); %Fluid temperature at the hot edge of the bed.  This temp is extrapolated from the last two nodes in the bed
[mufHedg,kfHedg,cfHedg,rhof,hfHedg,sfHedg]=fluidfun(THedg);
hflux(Nx+1,1)=sum(mdot.*hfCedg')*tau/Nt;
        
%Calculate system performance 
ref_load=0; heat_load=0;
for i=1:Nt
    if mdot(i)>0
        ref_load=ref_load-mdot(i)*(hfCedg(i)-hfC)*dt; %refrigeration in J supplied to the cold reservoir for one cycle by one bed
    end
    if mdot(i)<0
        heat_load=heat_load-mdot(i)*(hfHedg(i)-hfH)*dt; %heat transferred from the hot reservoir in J to the surroundings for one cycle by one bed
    end
end
ref_cap=ref_load/tau*n_beds %refrigeration rate in W supplied to the cold reservoir by the entire regenerator
heat_rej=heat_load*n_beds/tau %heat transfer rate from the hot reservoir in W to the surroundings by the entire regenerator
Wmag=heat_load-ref_load-Wpump; %magnetic work in J
Wmotor=Wmag/n_motor % Work of the motor on J done by the electric motor for each bed
Qpump=Wpump/(n_pump*tau)*n_beds; %rate of work applied to the pump in W for all beds in the regenerator
COP=ref_load/(Wmotor+Wpump/n_pump) %coefficient of performance
COP_Carnot=(TC)/(TH-TC); %Carnot efficiency for a cycle operating between the specified hot and cold reservoir temperatures

%% plot selected model results, if modelcheck is set to a value of 1.
if modelcheck==1
    resplot(t,xf,Tr,Tf,mdot,muoH,dwell,rhof)
end
