%% This function calculates the properties of the magnetocaloric material
%% across the entire bed.  The bed may be composed of multiple layers and
%% each layer may occupy a different fraction of the length of the bed.
%% The range of nodes occupied by each layer is calculated and the
%% properties for each layer are then calculated by calling the appropriate
%% material property function.  numlayer is the number of layers in the bed
%% The array inputs is an array with dimension numlayer x 2.  The array
%% contains the following variables: TCurie and Lfrac.  TCurie is the first
%% column of inputs and Lfrac is the second column.  The order of the
%% layers in inputs goes from x=0 to x=L.  For example, the first layer
%% listed should occupy x=0 to x=Lfrac(1) in the regenerator.  The layers
%% must be put in order in the inputs array and mcmfunc cell array.
%% TCurie is a row vector that contains the Curie temperature of mcm material
%%  in each layer, TCurie must have numlayer elements in the array.  The
%%  units are K.
%% Lfrac is a row vector that contains the portion of the length of the
%%  regenerator bed occupied by each layer.  Lfrac must have numlayer
%%  elements in the array and they must add to zero.  Each element must be
%%  in the range 0-1.
%% mcmfunc is a cell array that contains the function handle for the
%%  material property function used to evaluate the properties of mcm in
%%  each layer.

function[cmuoH,dsdmuoH,rhor,kr,sr,hr]=mcmprops(x,L,Tr,muoH,numlayer,inputs,mcmfunc);

%% parse the inputs array

TCurie=inputs(:,1);
Lfrac=inputs(:,2);
[Nx a]=size(x);

% cmuoH=zeros(size(Tr));
% dsdmuoH=zeros(size(Tr));
% sr=zeros(size(Tr));
% hr=zeros(size(Tr));

%% check inputs for errors

sumfrac=sum(Lfrac);
if sumfrac ~= 1
    fprintf('Fraction of Layers does not sum to 1')
end

if numlayer>Nx
    fprintf('Number of layers cannot be greater than number of spatial nodes')
end

[NT a]=size(TCurie);
[NL a]=size(Lfrac);
[Nmcm a]=size(mcmfunc);

if (NT ~= numlayer) || (NL ~= numlayer)
    fprintf('Number of elements in inputs does not equal number of layers\n')
end

%% if only one mcmfunction is given, the function assumes that every layer
%% is made of the same material

if Nmcm==1
    mcm=mcmfunc;
    for i=1:numlayer
        mcmfunc{i,1}=mcm{1};
    end
end

[Nmcm a]=size(mcmfunc);
if (numlayer ~= Nmcm)
    fprintf('Number of mcm functions does not equal number of layers\n')
end

%% calculate material properties for each layer, 1 - numlayer

xfrac=x/L; %dimensionless regenerator position
xlayer(1)=0; %x position of first layer boundary
j=0;
for i=1:numlayer
    j=j+1;
    numstart(i)=j; %node number of starting position of layer i
    xlayer(i+1)=xlayer(i)+Lfrac(i)*L; %x position of i+1 layer boundary
    while (x(j)<=xlayer(i+1)) & (j<Nx)
        j=j+1;
    end
    numfin(i)=j; %node number of ending position of layer i
    mcmfunction=mcmfunc{i}; %get mcm material function for node i
    Tri=Tr(numstart(i):numfin(i),:); %Tri is an array that contains the regenerator temperatures that will be sent to the mcm property function
    muoHi=muoH(numstart(i):numfin(i),:); %muoHi is an array that contains the magnetic field that will be sent to the mcm property function
    TCuriei=TCurie(i)*ones(numfin(i),1); %TCuriei is an array that contains the local Curie temperature that will be sent to the mcm property function
    [cmuoH(numstart(i):numfin(i),:),dsdmuoH(numstart(i):numfin(i),:),rholayer,klayer,sr(numstart(i):numfin(i),:),hr(numstart(i):numfin(i),:)]=mcmfunction(Tri,TCuriei,muoHi);
    rhor(numstart(i):numfin(i))=rholayer;
    kr(numstart(i):numfin(i))=klayer;
end
kr=kr';
rhor=rhor';


