%This function calculates the magnetic field assuming a trapezoidal
%magnetization curve and that the magnetic field is uniform across the
%regenerator bed at any instant in time.  At the beginning of the cycle,
%the magnetic field starts at 0 Tesla and begins to ramp up.  The field
%ramps up, then levels off to the maximum field value, ramps down, and ends
%with a magnetic field of 0 Tesla.
%The inputs are described below
%x      spatial grid of the regenerator bed
%t      temporal grid of the cycle (time)
%tau    time take for one complete cycle
%muoH_max   maximum magnetic field
%muoH_inp   array that contains the following variables 
%   1 arc_mag   arc occupied by the magnet (degrees)
%   2 arc_ramp  arc over which the magnetic field ramps to zero (degrees)
%
%The model calculates the magnetic field for the first spatial node in the
%bed and assigns that calculated value to all other nodes in the bed for
%each time step.  The length of the ramp length are assumed symmetric
%(equal ramp distance going from high to low as from low to high) and is
%variable.

function[muoH]=muoHfunction_ramp2(x,t,tau,muoH_max,muoH_inp);

[Nx,g]=size(x);
[Nt,g]=size(t);
arc_mag=muoH_inp(1);
arc_ramp=muoH_inp(2);

angle=t/tau*360; %convert time in cycle to an angle from 0-360
ang0=0; %beginning of ramp from low field to high field
ang1=arc_ramp+ang0; %beginning of flat high field section
ang2=arc_mag+ang1; %beginning of ramp from high field to low field
ang3=arc_ramp+ang2; %end of ramp from high field to low field

    for j=1:Nt
        if(angle(j)<ang1); %ramp up
            muoH(1,j)=muoH_max*(angle(j)-ang0)/arc_ramp;
        elseif(angle(j)>=ang1)&(angle(j)<=ang2); %high field
            muoH(1,j)=muoH_max;
        elseif(angle(j)>ang2)&(angle(j)<=ang3); %ramp down
            muoH(1,j)=muoH_max*(1-(angle(j)-ang2)/arc_ramp);
        else %zero field
            muoH(1,j)=0;
        end
        muoH(2:Nx,j)=muoH(1,j);
    end
