%% This function calculates model parameters for several packing geometries
%% It accepts an input array named inputs and calculates the appropriate
%% hydraulic diameter, porosity and specific surface area.  The array
%% 'inputs' contains different information about the regenerator geometry
%% for each type of regenerator.  The order of variables in 'inputs' is  
%%
%% For a packed sphere regenerator: 1. volumetric porosity
%%      2. dummy variable
%%      3. particle diameter
%%      4. dummy variable
%% 
%% For other regenerator geometries: 1. Dx, characteristic length in x-dir
%%      2. Px, pitch in x-direction
%%      3. Dy, characteristic length in y-direction
%%      4. Py, pitch in y-direction

function[dh,eps,epsX,epsY,as,FAR,PAR]=pack_geom(inputs,bedcorrfun);
 
geometry=func2str(bedcorrfun);

if (strcmp(geometry,'SphPart2')==1)%this is for sph_part

    %% assign variable to elements in the inputs array

    eps=inputs(1);
    dp=inputs(3);
    epsX=0;
    epsY=eps;

    %% calculate geometry-specific parameters

    dh=2/3*eps/(1-eps)*dp %m
    as=6*(1-eps)/dp %m^2/m^3
    FAR=1; %assign arbitrary number
    PAR=FAR*(1/epsY-1); %consistent with analytical model definition of PAR and FAR

elseif(strcmp(geometry,'AlignRect')==1)%this is for alignRect

    %%parse the input variable

    Dx=inputs(1);
    Px=inputs(2);
    Dy=inputs(3);
    Py=inputs(4);
    %processing...
    epsX=1-Dx/Px;
    epsY=Dy/Py;
    as=2*((Dx+Py-Dy)/(Px*Py))
    as=(1-epsX)*2*epsY/Dy
    eps=1-(Dx*(Py-Dy))/(Px*Py)
    dh=4*eps/as
    FAR=Dx/(Py-Dy);
    PAR=FAR*(1/epsY-1);
    %packing for output
%     gvar{1}(1)=dh;      names_gvar{1}='dh';
%     gvar{2}(1)=aw;      names_gvar{2}='aw';
%     gvar{3}(1)=epsilon; names_gvar{3}='epsilon';
%     gvar{4}(1)=Eyz;     names_gvar{4}='Eyz';
%     gvar{5}(1)=Ex;      names_gvar{5}='Ex';
%     gvar{6}(1)=FAR;     names_gvar{6}='FAR';
else
    error('geometry not yet coded or mismatch error with geometry and gvar inputs');
end

    
% if (strcmp(geometry,'sph_part')==1)%this is for sph_part
%     %identification
%     Dp=gvarin{1}(1);
%     epsilon=gvarin{2}(1);
%     %processing...
%     dh=Dp*(2/3)*(epsilon/(1-epsilon));
%     aw=6*(1-epsilon)/Dp;    
%     %packing for output
%     gvar{1}(1)=dh;      names_gvar{1}='dh';
%     gvar{2}(1)=aw;      names_gvar{2}='aw';
%     gvar{3}(1)=epsilon; names_gvar{3}='epsilon';
%     %...
% elseif (strcmp(geometry,'alignRect')==1)%this is for alignRect
%     %identification
%     Dx=gvarin{1}(1);
%     Px=gvarin{2}(1);
%     Dy=gvarin{3}(1);
%     Py=gvarin{4}(1);
%     %processing...
%     aw=2*((Dx+Py-Dy)/(Px*Py));
%     epsilon=1-(Dx*(Py-Dy))/(Px*Py);
%     dh=4*epsilon/aw;
%     Eyz=Dy/Py;
%     Ex=1-Dx/Px;
%     FAR=Dx/(Py-Dy);
%     %packing for output
%     gvar{1}(1)=dh;      names_gvar{1}='dh';
%     gvar{2}(1)=aw;      names_gvar{2}='aw';
%     gvar{3}(1)=epsilon; names_gvar{3}='epsilon';
%     gvar{4}(1)=Eyz;     names_gvar{4}='Eyz';
%     gvar{5}(1)=Ex;      names_gvar{5}='Ex';
%     gvar{6}(1)=FAR;     names_gvar{6}='FAR';
%     %...
% elseif (strcmp(geometry,'traps')==1)
%     Dx=     gvarin{1};
%     Px=     gvarin{2};
%     Py=     gvarin{3};
%     Dymin=  gvarin{4};
%     Dymax=  gvarin{5};
%     DySh=   gvarin{6};
%     DyMSa=  gvarin{7};
%     angle=  gvarin{8};
%     aff=    gvarin{9};
%     bff=    gvarin{10};
%     aNu=    gvarin{11};
%     bNu=    gvarin{12};
%     ea=     gvarin{13};
%     bNk=    gvarin{14};
%     v0low=  gvarin{15};
%     v0high= gvarin{16};% I only do this for the code to be more transparent,
%     %processing
%     awi=2*((sqrt(Dx.^2 + (Dymax-Dymin).^2) + Py - Dymax + DySh)./(Px.*Py));   
%     if (angle(1)==0)&&(angle(2)==90) || (angle(1)==90)&&(angle(2)==0)
%         dai=1;
%     else
%         dai=0;
%     end
%     sumtop=sum(awi.*Px - dai.*(1-Dymax./Py).*(1-Dymax./Py));
%     sumPx=sum(Px);
%     aw=sumtop/sumPx;
%     sumNum=sum(Dx.*(1-(Dymin+Dymax)./(2*Py)));
%     epsilon=1-(sumNum)/(sumPx);
%     dh=4*epsilon/aw;
%     Eyz=min(Dymin./Py);
%     sumPxminusDx=sum(Px-Dx);
%     Ex=sumPxminusDx/sumPx;
%     Dy=(Dymin+Dymax)/2;
%     sumDxPx=sum(Dx.*Px);
%     sumPyminusDyminPx= sum((Py-Dymin).*Px);
%     FAR=sumDxPx/(sumPyminusDyminPx);
%     %packing for output
%     gvar{1}(1)= dh;         names_gvar{1}='dh';
%     gvar{2}(1)= aw;         names_gvar{2}='aw';
%     gvar{3}(1)= epsilon;    names_gvar{3}='epsilon';
%     gvar{4}(1)= Eyz;        names_gvar{4}='Eyz';
%     gvar{5}(1)= Ex;         names_gvar{5}='Ex';
%     gvar{6}(1)= FAR;        names_gvar{6}='FAR';
%     gvar{7}(1)= aff;        names_gvar{7}='aff';
%     gvar{8}(1)= bff;        names_gvar{8}='bff';
%     gvar{9}(1)= aNu;        names_gvar{9}='aNu';
%     gvar{10}(1)=bNu;        names_gvar{10}='bNu';
%     gvar{11}(1)=ea;         names_gvar{11}='ea';
%     gvar{12}(1)=bNk;        names_gvar{12}='bNk';
%     gvar{13}(1)=v0low;      names_gvar{13}='v0low';
%     gvar{14}(1)=v0high;     names_gvar{14}='vohigh';
% else 
%     error('geometry not yet coded or mismatch error with geometry and gvar inputs');
% end

